<?php
/*
  TCNG: The Cancer Network Galaxy
  TCNG Main PHP routines.
  
  Copyright (C) 2012-2014 SiGN Project Members.
  Written by Yoshinori Tamada <tamada AT is.s.u-tokyo.ac.jp>
  
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as
  published by the Free Software Foundation, either version 3 of the
  License, or (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Affero General Public License for more details.
  
  You should have received a copy of the GNU Affero General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.   
*/

include 'conf.php';

function show_empty_search_box(){
  print '<div class="search">';
  print '<form method="GET" action="index.html" name="search">';
  print '<input id="query" type="text" name="q" value="" placeholder="search query" autofocus size="50%" /> ';
  print '<input id="submit" type="submit" name="" value="search" />';
  print '</form>';
  print '</div>';
}
/* show_empty_search_box() ================================================= */

function main($get){
  global $dbfile;
  if(! $db = new PDO("sqlite:" . $dbfile)){
	die("ERROR: DB Connection Failed.");
  }
  switch($get['t']){
  case 'network':
	show_network($db,$get);
	break;
  case 'nodelist':
	show_nodelist($db,$get);
	break;
  case 'gene':
	$id = $get['id'];
	include("header.php");
	put_search_box("");
	show_gene($db,$id);
	include("footer.php");
	break;
  case 'node':
	$id = $get['id'];
	include("header.php");
	put_search_box("");
	show_node($db,$id,$get);
	include("footer.php");
	break;
  case 'edge':
	include("header.php");
	put_search_box("");
	show_edge($db,$get);
	include("footer.php");
	break;
  case 'edgelist':
	show_edgelist($db,$get);
	break; 
  case 'subnetwork':
	show_subnetwork_json($db, $get);
	break;
  case 'geneset':
	$id = $get['id'];
	include("header.php");
	put_search_box("");
	show_gene_set($db,$id);
	include("footer.php");
	break;
  case 'networklist':
	show_network_list($db);
	break;
  case 'status':
	show_status($db);
	break;
  default:
	search($db,$get);
	break;
  }
}
/* main() ================================================================== */

/*
  Called from search().
*/
function parse_target(&$get){
  //echo 'q=' . $get['q'] . '<br>';
  $list = split(' ', $get['q']);
  $ret = array();
  foreach($list as $token){
	if(strcasecmp($token, 'target:edge') == 0 || strcasecmp($token, 't:edge') == 0){
	  $get['t'] = 'edge';
	} else {
	  $ret[] = $token;
	}
  }
  //var_dump($get);
  return $ret;
}
/* parse_target() ========================================================== */

function generate_query($db,$field,$qlist){
  $ret = NULL;
  $op = NULL;
  $ret = NULL;
  foreach($qlist as $q){
	if(strcasecmp($q,'or') == 0){
	  if($ret == NULL) return NULL; /* Syntax Error */
	  $op = 'OR';
	} else if(strcasecmp($q, 'and') == 0){
	  if($ret == NULL) return NULL; /* Syntax Error */
	  $op = 'AND';
	} else {
	  if($ret != NULL) $ret = '(' . $ret . ')';
	  if($ret != NULL && $op == NULL) $op = 'AND';
	  $ret .= " $op $field like " . $db->quote("%" . $q . "%");
	  $op = NULL;
	}
  }
  if($op != NULL) return NULL; /* Syntax Error */
  return $ret;
}
/* generate_query() ======================================================== */

function search($db,$get){
  $qlist = parse_target($get);
  //var_dump($get);
  //echo 'qlist[t] = ' . $qlist['t'];
  switch($get['t']){
  case 'edge':
	edge_search($db,$get,$qlist);
	break;
  case NULL:
  default:
	free_search($db,$get,$qlist);
	break;
  }
}
/* search() ================================================================ */

function parse($qlist){
  $r = array();
  foreach($qlist as $q){
	$elm = split(':', $q);
	if(count($elm) == 1){
	  $r['q'][] = $elm[0];
	} else if(count($elm) == 2){
	  $r[$elm[0]][] = $elm[1];
	}
  }
  return $r;
}
/* parse() ================================================================= */

function edge_search($db,$get,$qlist){
  $com = parse($qlist);

  $q = NULL;

  if($com['pa'] != NULL){
	$q1 = generate_query($db, 'n1.description', $com['pa']);
	$q2 = generate_query($db, 'n1.name', $com['pa']);
	if($q != NULL) $q .= ' AND';
	$q .= ' (' . $q1 . ' OR ' . $q2 . ')';
  }
  if($com['ch'] != NULL){
	$q1 = generate_query($db, 'n2.description', $com['ch']);
	$q2 = generate_query($db, 'n2.name', $com['ch']);
	if($q != NULL) $q .= ' AND';
	$q .= ' (' . $q1 . ' OR ' . $q2 . ')';
  }
  if($com['q'] != NULL){
	$q1 = generate_query($db, 'n1.description', $com['q']);
	$q2 = generate_query($db, 'n1.name', $com['q']);
	$q3 = generate_query($db, 'n2.description', $com['q']);
	$q4 = generate_query($db, 'n2.name', $com['q']);
	if($q != NULL) $q .= ' AND';
	$q .= ' (' . $q1 . ' OR ' . $q2 . ' OR ' . $q3 . ' OR ' . $q4 . ')';
  }
  if($q != NULL) $q = ' AND ' . $q;
  $q = 'SELECT n1.name n1nm, n1.id n1id, n2.name n2nm, n2.id n2id, edge.id eid'
	.   ' FROM edge,node n1,node n2'
	.   ' WHERE edge.node1=n1id AND edge.node2=n2id' . $q;
  //print $q;

  $r = $db->query($q)->fetchAll();
  $get['edge'] = $r;
  
  switch($get['fmt']){
  case NULL:
	include("header.php");
	put_search_box($get['q']);
	print '<h1>Edge Search Result</h1>';
	print '<table>';
	foreach($get['edge'] as $e){
	  //echo $e['n1nm'] . '-&gt;' . $e['n2nm'] . '(edge id=' . $e['eid'] . ')<br/>';
	  echo '<tr><td><a href="index.html?t=edge&id=' . $e['eid'] . '&id1=' . $e['n1id'] . '&id2=' . $e['n2id'] . '">'
	  . $e['n1nm'] . ' &rarr; ' . $e['n2nm'] . '</a></td></tr>';
	}
	print '</table>';
	include("footer.php");
	break;
  }
}
/* edge_search() =========================================================== */

function free_search($db,$get,$qlist){
  $q1 = generate_query($db,'gene_set.description', $qlist);
  $q2 = generate_query($db,'gene_set.name', $qlist);
  $q = "select network.id,gene_set.name,gene_set.description,array_set.ref,method.print as method "
	.    "from network,dataset,gene_set,array_set,method_param,method "
	.    "where network.dataset = dataset.id AND "
	.       "dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id AND "
	.       " network.method_param = method_param.id AND method_param.method = method.id AND "
	.          "(($q1) OR ($q2))";
  //echo "$q<br>";
  $networks = $db->query($q)->fetchAll();

  $q1 = generate_query($db,'name', $qlist);
  $q2 = generate_query($db,'description', $qlist);
  $q = "select * from gene where (($q1) OR ($q2))";
  //echo "$q<br>";
  $genes = $db->query($q)->fetchAll();

  switch($get['fmt']){
  case 'xml':
	header("Content-Type: text/xml; charset=UTF-8");
	print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
	print "<tcng_search_result>\n";
	print "<query>";
	print $get['q'];
	print "</query>\n";
	print "<result>\n";
	foreach($networks as $row){
	  print "<network>\n";
	  print "<id>" . $row['id'] . "</id>\n";
	  print "<name>" . $row['name'] . ' - ' . $row['ref'] . ' - ' . $row['method'] . "</name>\n";
	  print "<description>" . $row['description'] . "</description>\n";
	  print "</network>\n";
	}
	foreach($genes as $gene){
	  print "<gene>";
	  print '<id>' . $gene['id'] . "</id>\n";
	  print '<name>' . $gene['name'] . "</name>\n";
	  print '<description>' . $gene['description'] . "</description>\n";
	  print "</gene>";
	}
	print "</result>";
	print "</tcng_search_result>\n";
	break;

  case 'json':
	header('Content-type: application/json');
	$ret = array();
	$ret['query'] = $get['q'];
	$nets = array();
	foreach($networks as $row){
	  $net = array();
	  $net['id'] = $row['id'];
	  $net['name'] = $row['name'] . ' - ' . $row['ref'] . ' - ' . $row['method'];
	  $net['description'] = $row['description'];
	  $nets[] = $net;
	}
	if(count($nets) > 0) $ret['networks'] = $nets;
	$gens = array();
	foreach($genes as $row){
	  $gene = array();
	  $gene['id'] = $row['id'];
	  $gene['name'] = $row['name'];
	  $gene['description'] = $row['description'];
	  $gens[] = $gene;
	}
	if(count($gens) > 0) $ret['genes'] = $gens;
	echo json_encode($ret);
	break;

  case NULL:
  case 'html':
  default:
	include("header.php");
	put_search_box($get['q']);
	
	print '<div class="results">' . "\n";
	if(count($networks) != 0){
	  print '<h1>Matched gene networks: <span class="tagline">' . count($networks) . "</span></h1>\n";
	  print '<table>';
	  foreach($networks as $row){
		print '<tr><td>';
		print '<a href="index.html?t=network&id=' . $row['id'] . '">' 
		  . $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		  . "</a></td>";
		print '<td>' . $row['description'] . '</td></tr>' . "\n";
	  }
	  print '</table>';
	}
	if(count($genes) != 0){
	  print '<h1>Matched genes: <span class="tagline">' . count($genes) . "</span></h1>\n";
	  print '<table>';
	  foreach($genes as $row){
		print '<tr><td>';
		print '<a href="index.html?t=gene&id=' . $row['id'] . '">'
		  . $row['name'] . "</a></td><td>" . $row['description'] . "</td></tr>\n";
	  }
	  print '</table>';
	}

	if(count($networks) == 0 && count($genes) == 0){
	  print '<h1>No results found</h1>';
	}
	print "</div>\n"; /* results */
	include("footer.php");
	break;
  }/* switch($get['fmt']) */

}
/* free_search() =========================================================== */



function unparse($com){
  $ret = '';
  foreach(array_keys($com) as $key){
    if($ret != '') $ret .= '+';
    $ret .= $key . '%3a' . $com[$key];
  }
  return $ret;
}
/* unparse() =============================================================== */

function show_edgelist($db,$get){
  $qs = '';
  $id=$get['node'];
  if($id){
    $q = 'SELECT node.name FROM node WHERE node.id=' . $get['node'];
    $r = $db->query($q);
    $get['nodename'] = $r->fetchColumn();
  }
  if($get['network']){
    $q = 'SELECT gene_set.name,gene_set.description,method.print as method,array_set.ref'
	.      ' FROM network,dataset,gene_set,array_set,method_param,method'
	.      ' WHERE network.dataset = dataset.id'
	.         ' AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id'
	.         ' AND network.method_param = method_param.id AND method_param.method = method.id'
	.         ' AND network.id = ' . $get['network'];
    $r = $db->query($q)->fetchAll();
    $get['net'] = $r[0];

    if($qs != '') $qs .= ' AND ';
    $qs .= 'edge.network=' . $get['network'];
  }

  if($qs != '') $qs = ' AND (' . $qs . ')';
  $q = 'SELECT node.name,node.id nid,edge.id FROM edge,node'
	.     ' WHERE edge.node1=node.id AND edge.node2=' . $id . $qs;
  $get['up'] = $db->query($q);

  $q = 'SELECT node.name,node.id nid,edge.id FROM edge,node'
	.    ' WHERE edge.node2=node.id AND edge.node1=' . $id . $qs;
  $get['down'] = $db->query($q);

  switch($get['fmt']){
  case 'xml':
	show_edgelist_xml($db,$get);
	break;
  case 'json':
    show_edgelist_json($db,$get);
	break;
  case NULL:
  case 'html':
        include("header.php");
	put_search_box("");
	show_edgelist_html($db,$get);
	include("footer.php");
	break;
  }
}
/* show_edgelist() ========================================================= */

function show_edgelist_xml($db,$get){
  header("Content-Type: text/xml; charset=UTF-8");
  print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  print '<edgelist>';
  if($get['net']){
    $row = $get['net'];
    print '<network>';
    print '<id>' . $get['network'] . '</id>';
    print '<name>' . $row['name'] . " - " . $row['ref'] . " - " . $row['method'] . '</name>';
    print '<description>' . $row['description'] . '</description>';
    print '</network>';
  }
  print '<node>';
  print '<id>' . $get['node'] . '</id>';
  print '<name>' . $get['nodename'] . '</name>';
  print '</node>';

  print '<upstream>';
  foreach($get['up'] as $e){
    print '<node>';
    print '<name>' . $e['name'] . '</name>';
    print '<id>' . $e['nid'] . '</id>';
    print '<edge><id>' . $e['id'] . '</id></edge>';
    print '</node>';
  }
  print '</upstream>';
  print '<downstream>';
  foreach($get['down'] as $e){
    print '<node>';
    print '<name>' . $e['name'] . '</name>';
    print '<id>' . $e['nid'] . '</id>';
    print '<edge><id>' . $e['id'] . '</id></edge>';
    print '</node>';
  }
  print '</downstream>';
  
  print '</edgelist>';
}
/* show_edgelist_xml() ===================================================== */

function show_edgelist_json($db,$get){
  header('Content-type: application/json');
  $table = array();
  $nodes = array();
  $links = array();

  $id = $get['node'];
  $name = $get['nodename'];

  $table[$id] = true;
  $nodes[] = array('id' => $id, 'name' => $name);

  foreach($get['up'] as $e){
	$nid = $e['nid'];
	$link = array();
	$link['source'] = $nid;
	$link['target'] = $id;
	$link['id'] = $e['id'];
	if($table[$nid] == NULL){
	  $table[$nid] = true;
	  $nodes[] = array('id' => $nid, 'name' => $e['name']);
	}
  }
  foreach($get['down'] as $e){
	$nid = $e['nid'];
	$link = array();
	$link['source'] = $id;
	$link['target'] = $nid;
	$link['id'] = $e['id'];
	if($table[$nid] == NULL){
	  $table[$nid] = true;
	  $nodes[] = array('id' => $nid, 'name' => $e['name']);
	}
	$links[] = $link;
  }
  $ret['network'] = array('nodes' => $nodes,
						  'links' => $links);

  if($get['net']){
    $row = $get['net'];
	$ret['type'] = 'network';
	$ret['id'] = $get['network'];
	$ret['name'] = $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
	$ret['description'] = $row['description'];
  }
  $ret['condition'] = 'subnetwork';
  $ret['center'] = array('id' => $id, 'name' => $name);
  echo json_encode($ret);
  //var_dump($ret);
}
/* show_edgelist_json() ==================================================== */

function show_edgelist_html($db,$com){
  $id = $com['node'];
  $name = $com['nodename'];
  print "<h1>Edges in Network</h1>\n";

  print '<div class="card"><table>';
  if($com['net']){
    $row = $com['net'];
	print '<tr><td>Network</td><td>';
    print '<a href="index.html?t=network&id=' . $com['network'] . '">' 
      . $row['name'] . " - " . $row['ref'] . " - " . $row['method']
      . '</a></td></tr>';
	print '<tr><td>Network Description</td><td>';
	print $row['description'];
	print '</td></tr>';
  }
  if($name){
	print '<tr><td>Node</td><td>';
    print '<a href="index.html?t=node&id=' . $com['node'] . '">' 
      . $com['nodename'] . "</a></td>";
	print '</tr>';
  } else {
    print '<p><b>ERROR: Node ID required</b></p>';
    return;
  }
  print '</table></div>';

  print '<table class="node"><tr><th>Upstream (Parents)</th></tr>';
  $count = 0;
  $ar = array();
  foreach($com['up'] as $e){
	if(!$ar[$e['nid']]){
	  $ar[$e['nid']] = true;
	  print '<tr><td>';
	  print '&nbsp;&nbsp;&nbsp;(<a href="index.html?t=edgelist'
		. '&node=' . $e['nid'];
	  if($com['network']) print '&network=' . $com['network'];
	  print  '">&lt;&lt;</a>)&nbsp;&nbsp;&nbsp;&nbsp;'
		. '<a href="index.html?t=edge'
		. '&id1=' . $e['nid'] . '&id2=' . $com['node'];
	  if($com['network']) print '&network=' . $com['network'] . '&id=' . $e['id'];
	  print '">' . $e['name'] . ' &rarr; ' . $name . "</a></td></tr>\n";
	  $count++;
	}
  }/* for each $es */
  if($count == 0) print "<tr><td align=\"center\">(no edges)</td></tr>\n";
  print "</table>\n";

  print '<table class="node"><tr><th>Downstream (Children)</th></tr>';
  $count = 0;
  $ar = array();
  foreach($com['down'] as $e){
	if(!$ar[$e['nid']]){
	  $ar[$e['nid']] = true;
	  print '<tr><td>';
	  print '&nbsp;&nbsp;&nbsp;<a href="index.html?t=edge'
		. '&id1=' . $com['node'] . '&id2=' . $e['nid'];
	  if($com['network']) print '&network=' . $com['network'] . '&id=' . $e['id'];
	  print '">' . $name . ' &rarr; ' . $e['name'] . '</a>' 
		. '&nbsp;&nbsp;&nbsp;&nbsp;(<a href="index.html?t=edgelist'
		. '&node=' . $e['nid'];
	  if($com['network']) print '&network=' . $com['network'];
	  print '">&gt;&gt;</a>)'
		. "</td></tr>\n";
	  $count++;
	}
  }
  if($count == 0) print "<tr><td align=\"center\">(no edges)</td></tr>\n";
  print "</table>\n";

  print "<br/><br/>\n";
}
/* show_edgelist_html() ==================================================== */

function show_edge($db,$get){
  $id1 = $get['id1'];
  $id2 = $get['id2'];
  $net = $get['network'];
  if($net){
	print "<h1>Edge in Network</h1>\n";
  } else {
	print "<h1>Edge</h1>\n";
  }

  $q = 'SELECT name FROM node WHERE node.id=' . $id1;
  $r = $db->query($q)->fetchAll();
  $name1 = $r[0]['name'];
  $q = 'SELECT name FROM node WHERE node.id=' . $id2;
  $r = $db->query($q)->fetchAll();
  $name2 = $r[0]['name'];
  print "<h2><span class=\"tagline\">$name1 &rarr; $name2</span></h2>\n";
  print '<div class="results">';

  print '<div class="card"><table>';
  print '<tr><td>Parent Node</td><td><a href="index.html?t=node&id=' . $id1 . '">'
         . $name1 . "</a></td></tr>\n";
  $q = 'SELECT gene.id,gene.name FROM node_gene,gene WHERE node_gene.node_id=' . $id1
        . ' AND node_gene.gene_id = gene.id';
  $r = $db->query($q);
  print "<tr><td>Genes in Parent Node</td><td>";
  foreach($r as $g){
    print '<a href="index.html?t=gene&id=' . $g['id'] . '">' . $g['name'] . "</a><br/>\n";
  }
  print "</td></tr>\n";

  print '<tr><td>Child Node</td><td><a href="index.html?t=node&id=' . $id2 . '">'
         . $name2 . "</a></td></tr>\n";
  $q = 'SELECT gene.id,gene.name FROM node_gene,gene WHERE node_gene.node_id=' . $id2
        . ' AND node_gene.gene_id = gene.id';
  $r = $db->query($q);
  print "<tr><td>Genes in Child Node</td><td>";
  foreach($r as $g){
    print '<a href="index.html?t=gene&id=' . $g['id'] . '">' . $g['name'] . "</a><br/>\n";
  }
  print "</td></tr>\n";

  if($net){
    $q = 'SELECT gene_set.name,gene_set.description,array_set.ref,method.print as method'
	.      ' FROM network,dataset,gene_set,array_set,method_param,method,gene_set_node'
	.      ' WHERE network.dataset = dataset.id'
	.        ' AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id'
	.        ' AND network.method_param = method_param.id AND method_param.method = method.id'
	.        ' AND gene_set_node.gene_set = gene_set.id'
	  .        ' AND network.id = ' . $net;

    $nn = $db->query($q)->fetchAll();
    $row = $nn[0];
    print '<tr><td>Network</td><td><a href="index.html?t=network&id=' . $e['network'] . '">' 
		  . $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		  . "</a></td></tr><tr><td>Description</td><td>" . $row['description'] . "</td></tr>\n";
  }

  print "</table></div>\n";

  if($net){
	print '<h3>Edge Attributes</h3>';
	$q = 'SELECT name,type,value FROM edge_attr WHERE edge_attr.edge=' . $get['id'];
	$r = $db->query($q)->fetchAll();
	print '<table>';
	foreach($r as $a){
	  print '<tr><td><b>' . $a['name'] . '</b> (' . $a['type'] . ') : ' . $a['value'] . "</td></tr>\n";
	}
	print '</table>';
  } else {

	$q = 'SELECT edge.network network, edge.id id'
	  //.     ', edge_attr.name atname, edge_attr.type attype, edge_attr.value atval'
	  //.   ' FROM edge, edge_attr'
	  .   ' FROM edge'
	  //.   ' WHERE edge.id = edge_attr.edge'
	  .   ' WHERE'
	  .     ' node1=' . $id1 . ' AND node2=' . $id2;
	$r = $db->query($q);
	print "<h2>Networks Including this Edge</h2>\n";
	print '<table>';
	foreach($r as $e){
	  $q = 'SELECT gene_set.name,gene_set.description,array_set.ref,method.print as method'
		.      ' FROM network,dataset,gene_set,array_set,method_param,method,gene_set_node'
		.      ' WHERE network.dataset = dataset.id'
		.        ' AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id'
		.        ' AND network.method_param = method_param.id AND method_param.method = method.id'
		.        ' AND gene_set_node.gene_set = gene_set.id'
		.        ' AND network.id = ' . $e['network'];
	  
	  $nn = $db->query($q)->fetchAll();
	  $row = $nn[0];
	  print '<tr><td><a href="index.html?t=network&id=' . $e['network'] . '">' 
		. $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		. "</a></td><td>" . $row['description'] . "</td></tr>\n";
	  
	  /*
		print '&nbsp;&nbsp;&nbsp;<b>Edge Attributes</b><br/>';
		$q = 'SELECT name,type,value FROM edge_attr WHERE edge_attr.edge=' . $e['id'];
		$r = $db->query($q)->fetchAll();
		foreach($r as $a){
		print '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>' . $a['name'] . '</b> (' . $a['type'] . ') : ' . $a['value'] . "<br/>\n";
		}
	  */
	}
	print '</table>';
  }

  print '</div>';
  
  print "<br/><br/>\n";
}
/* show_edge() ============================================================= */

function show_node($db,$id,$com){
  print "<h1>Node</h1>\n";

  $q = 'SELECT name FROM node WHERE node.id=' . $id;
  $r = $db->query($q);
  $name = $r->fetchColumn();
  print '<h2><span class="tagline">' . $name . "</span></h2>";

  print '<a href="index.html?t=edgelist&node=' . $id . '">' 
	. '(Show all edges with this node)</a><br/>';

  print '<h2>Genes Involved in this Node</h2>';
  $q = 'SELECT gene.name,gene.id,gene.description'
         . ' FROM node,gene,node_gene'
	 . ' WHERE node.id=' . $id
	 .    ' AND node.id=node_gene.node_id AND gene.id = node_gene.gene_id';
  $r = $db->query($q)->fetchAll();
  print '<div class="card"><table>';
  foreach($r as $row){
	  print '<tr><td><a href="index.html?t=gene&id=' . $row['id'] . '">'
		. $row['name'] . "</a></td><td>" . $row['description'] . "</td></tr>";
	  
  }
  print '</table></div>' . "\n";

  print "<h2>Networks including this Node</h2>\n";
  print "<p>(Click on a network name to show matched edges)</p>\n";
  $q = 'SELECT network.id,gene_set.name,gene_set.description,array_set.ref,method.print as method'
	.      ' FROM network,dataset,gene_set,array_set,method_param,method,gene_set_node'
	.      ' WHERE network.dataset = dataset.id AND'
	.        ' dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id AND'
	.        ' network.method_param = method_param.id AND method_param.method = method.id AND'
	.        ' gene_set_node.gene_set = gene_set.id AND'
	.        ' gene_set_node.node=' . $id;
  $r = $db->query($q);
  $nets = array();
  print '<table>';
  foreach($r as $row){
	if(! $nets[$row['id']]){
	  $netid = $row['id'];
	  $nets[$row['id']] = true;
	  print '<tr><td><a href="index.html?t=edgelist&node=' . $id
		. '&network=' . $row['id'] . '">' 
		. $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		. "</a></td><td>" . $row['description'] . "</td></tr>\n";
	}
  }
  print '</table>';
  
  print '<br/><br/>';
}
/* show_node() ============================================================= */

function show_gene($db,$id){
  print "<h1>Gene</h1>\n";

  $q = 'SELECT gene.name,gene.description'
	.    ' FROM gene'
	.    ' WHERE gene.id=' . $id;
  $r = $db->query($q)->fetchAll();
  if(count($r) == 1){
	$row = $r[0];

	print '<h2><span class="tagline">' . $row['name'] . "</h2>";

	print '<div class="card"><table>';
	print "<tr><td>Entrez Gene ID</td><td>$id";
	print " (<a href=\"http://www.ncbi.nlm.nih.gov/gene/" . $id .
	  "\">See on NCBI</a>)</td></tr>\n";

	print "<tr><td>Description</td><td>";
	print $row['description'];
	print "</td></tr>\n";

	print "<tr><td>Gene Synonyms</td><td>\n";
	$q = 'SELECT name FROM synonym_gene WHERE synonym_gene.gene_id=' . $id;
	$r = $db->query($q);
	$flag = false;
	foreach($r as $row){
	  if($flag) print ', ';
	  print $row['name'];
	  $flag = true;
	}
	print "</td></tr>\n";

	$q = 'SELECT node.name, node.id'
	  .     ' FROM node,node_gene,gene'
	  .     ' WHERE node_gene.gene_id = gene.id AND node_gene.node_id = node.id'
	  .        ' AND gene.id =' . $id;
	$r = $db->query($q)->fetchAll();
	print "<tr><td>Node consisting of this Gene</td><td>";
	$node_ids = array();
	foreach($r as $n){
	  print '<a href="index.html?t=node&id=' . $n['id'] . '">' . $n['name'] . "</a><br/>\n";
	  if(! array_search($n['id'], $node_ids)){
		array_push($node_ids, $n['id']);
	  }
	}
	print "</td></tr>\n";
	print "</table></div>\n";

	print "<h2>Networks including this gene</h2>\n";
	$flag = false;
	$q = '';
	foreach($node_ids as $id){
	  if($flag) $q = $q . ' OR ';
	  $q = $q . 'gene_set_node.node=' . $id;
	  $flag = true;
	}
	$q = 'SELECT network.id,gene_set.name,gene_set.description,array_set.ref,method.print as method'
		.      ' FROM network,dataset,gene_set,array_set,method_param,method,gene_set_node'
		.      ' WHERE network.dataset = dataset.id AND'
		.        ' dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id AND'
		.        ' network.method_param = method_param.id AND method_param.method = method.id AND'
		.        ' gene_set_node.gene_set = gene_set.id AND'
		.        ' (' . $q . ')';
	$r = $db->query($q);
	$count=0;
	$nets = array();
	print '<table>';
	foreach($r as $row){
	  if(! $nets[$row['id']]){
  	    print '<tr><td><a href="index.html?t=network&id=' . $row['id'] . '">' 
		  . $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		  . "</a></td><td>" . $row['description'] . "</td></tr>\n";
 	    $nets[$row['id']] = true;
		$count++;
	  }
	}
	print '</table>';
	if($count == 1){
	  print '<p>(' . $count . ' network)</p>';
	} else if($count > 1){
	  print '<p>(' . $count . ' networks)</p>';
	}
  }
  
  print "<br/><br/>\n";
}
/* show_gene() ============================================================= */

function show_gene_set($db,$id){
  print "<h1>Gene Set</h1>\n";

  $q = 'SELECT gene_set.name as name, gene_set.description, gene_set_method.description as method,'
	.     ' gene_set.src_ref as ref, data_src.name as src'
	.     ' FROM gene_set, gene_set_method,data_src'
	.     ' WHERE gene_set.gene_set_method = gene_set_method.id'
	.         ' AND gene_set.src_type = data_src.id'
	.         ' AND gene_set.id=' . $id;
  $r = $db->query($q)->fetchAll();
  if(count($r) == 1){
	$row = $r[0];

	print '<h2><span class="tagline">' . $row['name'] . "</span></h3>\n";
	print '<div class="card"><table>';

	print "<tr><td>Description</td><td>";
	print $row['description'];
	print "</td></td>\n";

	print "<tr><td>Gene Set Method</td><td>";
	print $row['method'];
	print "</td></tr>\n";

	print "<tr><td>Gene Set Source</td><td>";
	print $row['src'] . '&nbsp;:&nbsp;' . $row['ref'];
	switch($row['src']){
	case 'GEO':
	  print ' (<a href="http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=' . $row['ref'] . '">See on GEO</a>)';
	  break;
	}
	print "</td></tr>\n";

	$q = 'select count(*) from gene_set_node where gene_set_node.gene_set=' . $id;
	$num = $db->query($q)->fetchColumn();
	print "<tr><td>Number of Nodes</td><td>" . $num;
	print "</td></tr>\n";

	print "</table></div>\n";


	$q = 'select node.id,node.name as name from node,gene_set_node'
	  .     ' WHERE gene_set_node.node = node.id'
	  .        ' AND gene_set_node.gene_set = ' . $id;
	$r = $db->query($q)->fetchAll();
	
	print "<h2>Network Nodes in the Gene Set</h2>\n";
	print '<table>';
	foreach($r as $row){
	  print '<tr><td><a href="index.html?t=node&id=' . $row['id'] . '">' . $row['name'] 
		. "</a></td></tr>\n";
	}
	print '</table>';
  }  
  print "<br/><br/>\n";
}
/* show_gene_set() ========================================================= */

function show_network_xml($r,$id,$num){
  header("Content-Type: text/xml; charset=UTF-8");
  print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
  if(count($r) == 1){
	$row = $r[0];
	print '<network>';
	print '<id>' . $id . '</id>';
	print '<name>';
	print $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
	print '</name>';

	print '<arrayset>' . $row['ref'] . '</arrayset>';
	print '<arrayset_source>' . $row['src'] . '</arrayset_source>';
	print '<geneset>';
	print '<name>' . $row['name'] . '</name>';
	print '<id>' . $row['gid'] . '</id>';
	print '<description>' . $row['description'] . '</description>';
	print '<num_nodes>' . $num . '</num_nodes>';
	print '</geneset>';
	print '<method>' . $row['method'] . '</method>';
	print '</network>';
  }
}
/* show_network_xml() ====================================================== */

function cmp_node_bn($a, $b){
  if($a['betweenness'] == $b['betweenness']) return 0;
  return ($a['betweenness'] > $b['betweenness']) ? -1 : 1;
}
/* cmp_node_bn() =========================================================== */

function cmp_node_pa($a, $b){
  if($a['num_parents'] == $b['num_parents']){
	if($a['num_children'] == $b['num_children']) return 0;
	return ($a['num_children'] > $b['num_children']) ? -1 : 1;
  }
  return ($a['num_parents'] > $b['num_parents']) ? -1 : 1;
}
/* cmp_node_bn() =========================================================== */

function cmp_node_deg($a, $b){
  if($a['degree'] == $b['degree']){
	if($a['num_children'] == $b['num_children']) return 0;
	return ($a['num_children'] > $b['num_children']) ? -1 : 1;
  }
  return ($a['degree'] > $b['degree']) ? -1 : 1;
}
/* cmp_node_bn() =========================================================== */

/*
  $r : query results
  $id : network id
*/
function show_network_json_num($db,$r,$id,$files,$num,$th,$sort){
  header('Content-type: application/json');
  $ret = array();
  if(count($r) == 1) $row = $r[0];

  /* Check files first */
  foreach($files as $f){
	$path = 'f/' . $row['release'] . '/' . $row['gsm'] . '/' . $row['mn'] . '/' . $row['mpn'] . '/'
	  . $f['ext'] . '/' . $f['name'];
	if($f['format'] == 'NODELIST'){
	  $nltxt = $path;
	} else if($f['format'] == 'text'){
	  $txt = $path;
	}
  }

  /* Get the number of nodes in a network */
  $q = 'SELECT count(*) FROM gene_set_node'
	.    ' WHERE gene_set_node.gene_set=' . $row['gid'];
  $totalnum = $db->query($q)->fetchColumn();

  /* put nodes */
  $fp = fopen($nltxt, 'r');
  if($fp == NULL) die("ERROR: NULL");
  $line = fgets($fp);

  $nodes = array();
  $table = array();
  $rank = 1;
  $max = (float) 0.0;
  $min = (float) $totalnum * $totalnum;
  while(!feof($fp)){
	$node = array();
	$token = split("\t", fgets($fp));
	if(count($token) < 5) break;
	if($max < $token[5]) $max = (float) $token[5];
	if($token[5] < $min) $min = (float) $token[5];
	$q = 'SELECT id from node where name=' . $db->quote($token[0]);
	$r = $db->query($q);
	$nid = $r->fetchColumn();
	$node['id'] = $nid;
	$node['name'] = $token[0];
	$node['betweenness'] = round($token[5],3);
	$node['norm_betweenness'] = (float) $token[5];
	$node['hub_rank'] = $rank;
	$node['num_children'] = (int) $token[2];
	$node['num_parents'] = (int) $token[3];
	$node['degree'] = (int) $token[4];
	$nodes[] = $node;

	$table[$token[0]] = & $nodes[$rank - 1];
	$rank++;
  }

  /* Normalize betweenness */
  $range = $max - $min;
  foreach($nodes as &$n){
	$n['norm_betweenness'] = ($n['norm_betweenness'] - $min) / $range;
  }

  /* sort */
  if($sort == 'betweenness'){
	usort($nodes, "cmp_node_bn");
	$rank = 1;
	foreach($nodes as &$n){
	  $n['betweenness_rank'] = $rank;
	  $rank++;
	}
  } else if($sort == 'parents'){
	usort($nodes, "cmp_node_pa");
	$rank = 1;
	foreach($nodes as &$n){
	  $n['parents_rank'] = $rank;
	  $rank++;
	}
  } else if($sort == 'degree'){
	usort($nodes, "cmp_node_deg");
	$rank = 1;
	foreach($nodes as &$n){
	  $n['degree_rank'] = $rank;
	  $rank++;
	}
  }

  /* At first, add $num nodes from '$nodes' to '$table2' and '$table3'. */
  /* '$table3' is used to check if the node is used. */
  $table2 = array(); /* hub list */
  $table3 = array(); /* table for nodes added in $node2 */
  $nodes2 = array();
  for($i = 0; $i < $num; $i++){
    $table2[$nodes[$i]['name']] = true;

	$table3[$nodes[$i]['name']] = true;
	$nodes2[] = $table[$nodes[$i]['name']];
  }

  /* Open TXT file for edges */
  $fp = fopen($txt, 'r');
  if($fp == NULL) die("ERROR: NULL");
  $token = split("\t", fgets($fp));
  for($i = 0; $i < count($token); $i++){
	if($token[$i] == 'edgeScore'){
	  $pos = $i;
	  break;
	}
  }

  $links = array();

  /* Collect all edges first. */
  $all_links = array();
  while(!feof($fp)){
    $token = split("\t", fgets($fp));
    $e = array();
    $e[0] = $token[0];
    $e[1] = $token[1];
    $e[2] = $token[$pos];
    $all_links[] = $e;

    $link = array();
    $flag = false;
    if(($table2[$token[0]] != NULL) || ($table2[$token[1]] != NULL)){
      $flag = true;
    }

    if($flag && ($th < 0 || $th < floatval($token[$pos]))){
          $link = array();
	  $link['source'] = $table[$token[0]]['id'];
	  $link['target'] = $table[$token[1]]['id'];
	  $link['score'] = $token[$pos];
	  $links[] = $link;

	  if($table3[$token[0]] == NULL){
	    $table3[$token[0]] = true;
	    $nodes2[] = $table[$token[0]];
	  }
	  if($table3[$token[1]] == NULL){
	    $table3[$token[1]] = true;
            $nodes2[] = $table[$token[1]];
	  }
	}
  }/*while(!eof))*/

  /* Add edges connecting nodes in the sub network but not the ranked nodes. */
  foreach($all_links as $token){
    if(($table3[$token[0]] != NULL && $table3[$token[1]] != NULL) &&
       ($th < 0 || $th < floatval($token[2]))){
      $link = array();
      $link['source'] = $table[$token[0]]['id'];
      $link['target'] = $table[$token[1]]['id'];
      $link['score'] = $token[2];
      $links[] = $link;
    }
  }
  $network = array();
  $network['nodes'] = $nodes2;
  $network['links'] = $links;
  $ret['network'] = $network;
  $ret['type'] = 'network';
  $ret['id'] = $id;
  $ret['name'] = $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
  if($num > 0) $ret['condition'] = 'top ' . $num . ' hub nodes';
  if($th > 0) $ret['edge_threshold'] = $th;
  $ret['description'] = $row['description'];
  $ret['number_of_nodes'] = $totalnum;
  $ret['max_betweenness'] = $max;
  $ret['min_betweenness'] = $min;
  $ret['range_betweenness'] = $range;
  if($sort) $ret['order'] = $sort;
  echo json_encode($ret);
  //var_dump($ret);
}
/* show_network_json_num() ================================================= */

function show_network_json($db,$r,$id,$files,$th){
  header('Content-type: application/json');
  $ret = array();
  if(count($r) == 1) $row = $r[0];

  /* Check files first */
  foreach($files as $f){
	$path = 'f/' . $row['release'] . '/' . $row['gsm'] . '/' . $row['mn'] . '/' . $row['mpn'] . '/'
	  . $f['ext'] . '/' . $f['name'];
	if($f['format'] == 'NODELIST'){
	  $nltxt = $path;
	} else if($f['format'] == 'text'){
	  $txt = $path;
	}
  }

  /* Get the number of nodes in a network */
  $q = 'SELECT count(*) FROM gene_set_node'
	.    ' WHERE gene_set_node.gene_set=' . $row['gid'];
  $totalnum = $db->query($q)->fetchColumn();

  /* put nodes */
  $fp = fopen($nltxt, 'r');
  if($fp == NULL) die("ERROR: NULL");
  $line = fgets($fp);

  $table = array();
  $rank = 1;
  $max = (float) 0.0;
  $min = (float) $totalnum * $totalnum;
  while(!feof($fp)){
	$node = array();
	$token = split("\t", fgets($fp));
	if(count($token) < 5) break;
	if($max < $token[5]) $max = (float) $token[5];
	if($token[5] < $min) $min = (float) $token[5];
	$q = 'SELECT id from node where name=' . $db->quote($token[0]);
	$r = $db->query($q);
	$nid = $r->fetchColumn();
	$node['id'] = $nid;
	$node['name'] = $token[0];
	$node['betweenness'] = round($token[5],3);
	$node['norm_betweenness'] = (float) $token[5];
	$node['hub_rank'] = $rank;
	$node['num_children'] = (int) $token[2];
	$node['num_parents'] = (int) $token[3];
	$node['degree'] = (int) $token[4];
	$table[$token[0]] = $node;
	$rank++;
  }

  /* Normalize betweenness */
  $range = $max - $min;
  foreach($table as &$n){
	$n['norm_betweenness'] = ($n['norm_betweenness'] - $min) / $range;
  }

  /* put edges */
  $fp = fopen($txt, 'r');
  if($fp == NULL) die("ERROR: NULL");
  $token = split("\t", fgets($fp));
  for($i = 0; $i < count($token); $i++){
	if($token[$i] == 'edgeScore'){
	  $pos = $i;
	  break;
	}
  }

  $nodes = array();
  $links = array();
  $table4 = array();
  while(!feof($fp)){
	$token = split("\t", fgets($fp));
	if(count($token) < 2) continue;
	if($th < 0 || $th < floatval($token[$pos])){
	  $link = array();
	  $link['source'] = $table[$token[0]]['id'];
	  $link['target'] = $table[$token[1]]['id'];
	  $link['score'] = $token[$pos];
	  $links[] = $link;

	  if($table4[$token[0]] == NULL){
	    $table4[$token[0]] = true;
	    $nodes[] = $table[$token[0]];
	  }
	  if($table4[$token[1]] == NULL){
	    $table4[$token[1]] = true;
	    $nodes[] = $table[$token[1]];
	  }
        }
  }
  $network = array();
  $network['nodes'] = $nodes;
  $network['links'] = $links;
  $ret['network'] = $network;
  $ret['type'] = 'network';
  $ret['id'] = $id;
  $ret['name'] = $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
  if($th > 0) $ret['edge_threshold'] = $th;
  $ret['description'] = $row['description'];
  $ret['number_of_nodes'] = $totalnum;
  $ret['max_betweenness'] = $max;
  $ret['min_betweenness'] = $min;
  $ret['range_betweenness'] = $range;
  echo json_encode($ret);
  //var_dump($ret);
}
/* show_network_json() ===================================================== */

function show_network_html($r,$id,$num,$files,$db){
  include("header.php");
  put_search_box("");
  print '<h1>Gene Network</h1>';
  print '<div class="results">';
  
  if(count($r) == 1){
	$row = $r[0];
	
	print '<h2><span class="tagline">' 
	  . $row['name'] . " - " . $row['ref'] . " - " . $row['method'] . "</span></h2>\n";
	print '<div class="card">';
	print '<table>';
	print "<tr><td>Network Name</td><td>";
	print $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
	//	  . "</b> &nbsp;&nbsp;&nbsp; " . $row['description'];
	print "</td></tr>\n";

	print "<tr><td>Array Set</td><td>";
	print $row['ref'];
	switch($row['src']){
	case 'GEO':
	  print ' (<a href="http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=' . $row['ref'] . '">See on GEO</a>)';
	  break;
	}

	print "</td></tr>\n";

	print "<tr><td>Array Set Source</td><td>" . $row['src'] . "</td></tr>\n";

	print "<tr><td>Gene Set</td><td>"
	  . '<a href="index.html?t=geneset&id=' . $row['gid'] . '">'
	  . $row['name'] . "</a></td></tr>\n";

	print "<tr><td>Gene Set Description</td><td>" 
	  . $row['description'] . "</td></tr>\n";
	print "<tr><td>Number of Nodes</td><td>" . $num . "</td></tr>\n";
	print "<tr><td>Network Method</td><td>" . $row['method'] . "</td></tr>\n";
	print "<tr><td>Downloadble Files</td><td>";

	foreach($files as $f){
	  $path = 'f/' . $row['release'] . '/' . $row['gsm'] . '/' . $row['mn'] . '/' . $row['mpn'] . '/'
		. $f['ext'] . '/' . $f['name'];
	  print $f['format'] . ' format - ' 
		. '<a href="' . $path . '">'
		. $f['name'] . "</a><br/>\n";
	  if($f['format'] == 'NODELIST'){
		$nltxt = $path;
	  }
	}
	print "</td></tr>\n";
	print "</table></div>";

	if($nltxt){
	  print "<h3>Top Hub Genes in This Network</h3>\n";
	  print "(Click on a gene name to browse the network around the gene.)";
	  /*
	  print "<table><tr><th align=\"left\" >Rank</th><th align=\"left\">Node</th><th align=\"right\">Num. Children</th>"
		. "<th align=\"right\">&nbsp;&nbsp;&nbsp;&nbsp;Betweenness</th></tr>\n";
	  $fp = fopen($path,'r');
	  $line = fgets($fp);
	  $i = 1;
	  while($i <= 20 && !feof($fp)){
		$token = split("\t", fgets($fp));
		print "<tr><td>#" . $i . "</td>";
		$q = 'select id from node where name=' . $db->quote($token[0]);
		$r = $db->query($q);
		$nid = $r->fetchColumn();
		if($nid){
		  //print '<td><a href="index.html?t=node&id=' . $nid . '">'. $token[0] . "</a></td>";
		  print '<td><a href="index.html?t=edgelist&node=' . $nid . '&network=' . $id . '">'. $token[0] . '</a></td>';
		} else {
		  print "<td>" . $token[0] . "</td>";
		}
		print '<td align="right">' . $token[2] . "</td>";
		print '<td align="right">' . round($token[5],3) . "</td>";
		print "</tr>\n";
		$i++;
	  }
	  print "</table>";
	  */
	  print '<iframe class="nodelist" src="index.html?t=nodelist&id='.$id.'&p='.htmlspecialchars($path,ENT_QUOTES).'" ></iframe>';
	}
  } else {
	print "<b>ERROR - Unknown ID specified.</b>\n";
  }
  //print '<a href="index.html?t=nodelist&id='.$id.'&p='.htmlspecialchars($path,ENT_QUOTES).'">Show all nodes.</a>';
  //print "<br/><br/>";

	
  include("footer.php");
}
/* show_network_html() ===================================================== */

function show_network($db,$get){
  $id = $get['id'];

  $q = "select network.id,gene_set.name,gene_set.description,array_set.ref,method.print as method, "
	.    " release.name as release, gene_set_method.name as gsm, method.name as mn, method_param.name as mpn,"
	.    " gene_set.id as gid, data_src.name as src"
	.    " from network,dataset,gene_set,array_set,method_param,method,release,release_network,"
	.    " gene_set_method,data_src "
	.    " where network.dataset = dataset.id"
	.       " AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id"
	.       " AND network.method_param = method_param.id AND method_param.method = method.id"
	.       " AND release.id = release_network.release AND release_network.network = network.id"
	.       " AND gene_set_method.id = gene_set.gene_set_method AND data_src.id = array_set.data_src"
	.       " AND network.id=$id";
  $r = $db->query($q)->fetchAll();

  $num = NULL;
  $files = NULL;
  if(count($r) == 1){
	$q = 'select count(*) from gene_set_node where gene_set_node.gene_set=' . $r[0]['gid'];
	$num = $db->query($q)->fetchColumn();

	$q = 'select file.name,format.name as format,format.ext as ext'
	  .   ' from file,format '
	  .   ' where file.format = format.id AND file.network=' . $id;
	$files = $db->query($q)->fetchAll();
  }

  switch($get['fmt']){
  case 'xml':
	show_network_xml($r,$id,$num);
	break;
  case 'json':
	$th = $get['th'];
	if($th == NULL) $th = -1;
	if($get['num']){
	  show_network_json_num($db,$r,$id,$files,$get['num'],$th,$get['order']);
	} else {
	  show_network_json($db,$r,$id,$files,$th);
	}
	break;
  case NULL:
  case 'html':
	show_network_html($r,$id,$num,$files,$db);
	break;
  }
}
/* show_network() ========================================================== */

function show_nodelist($db,$get){
  $id = $get['id'];
  $path = $get['p'];

  $fp = fopen($path, 'r');
  $line = fgets($fp);
  print '<!DOCTYPE html>';
  print '<html><head>';
  print '<link rel="stylesheet" type="text/css" media="screen" href="style.css" />';
  print '<script type="text/javascript" src="sorter.js"></script>';
  print '</head>';
  print '<body><div id="container"><div class="nodelist"><div id="content">';
  //print "<h3>" . $path . "</h3>";
  if($fp == NULL) die("ERROR: NULL");

  /*
  print "<table class=\"nlbase\"><tr><td width=\"10%\">";
  print "<table width=\"10%\"><tr><th align=\"left\">Rank</th></tr>";
  $i = 1;
  $lines = array();
  while(!feof($fp)){
	$lines[] = fgets($fp);
	print '<tr><td>#' . $i . '</td></tr>' . "\n";
	$i++;
  }
  print '</table></td><td>';
  */
  print "<table id=\"nodelist\" class=\"sortable\"><tr>"
	. '<th align="left">Hub Rank</th>'
	. "<th align=\"left\">Node</th><th align=\"right\">Children</th>"
	. '<th align="right">Parents</th>'
	. '<th align="right">Degree</th>'
	. "<th align=\"right\">&nbsp;&nbsp;&nbsp;&nbsp;Betweenness</th></tr>\n";
  //while($i <= 10 && !feof($fp)){
  $i = 1;
	while(!feof($fp)){
  //for($i = 0; $i < count($lines); $i++){
	$token = split("\t", fgets($fp));
	if(count($token) < 5) break;
	//$token = split("\t", $lines[$i]);
	print "<tr><td>#" . $i . "</td>";
	$q = 'select id from node where name=' . $db->quote($token[0]);
	$r = $db->query($q);
	$nid = $r->fetchColumn();
	if($nid){
	  print '<td><a href="index.html?t=edgelist&node=' . $nid . '&network=' . $id . '" target="_parent">'. $token[0] . '</a></td>';
	} else {
	  print "<td>" . $token[0] . "</td>";
	}
	print '<td align="right">' . $token[2] . "</td>";
	print '<td align="right">' . $token[3] . "</td>";
	print '<td align="right">' . $token[4] . "</td>";
	print '<td align="right">' . round($token[5],3) . "</td>";
	print "</tr>\n";
	$i++;
  }

  print '</table>';
  print '<script type="text/javascript">';
  print 'var sorter=new table.sorter("sorter");';
  print 'sorter.init("nodelist",1);';
  print '</script>';

  //print '</td></tr></table>';
  print '</div></div></div></body></html>';
}
/* show_nodelist() ========================================================= */

function show_status($db){
  include("header.php");
  put_search_box('');
  print "<h1>TCNG Database Status</h1>";

  print "<table>\n";

  print "<tr><td>The Number of Gene Sets</td><td>";
  $q = 'SELECT count(*) FROM gene_set';
  $r = $db->query($q)->fetchColumn();
  print $r . "</td></tr>\n";

  print "<tr><td>The Number of Array Sets</td><td>";
  $q = 'SELECT count(*) FROM array_set';
  $r = $db->query($q)->fetchColumn();
  print $r . "</td></tr>\n";

  print "<tr><td>The Number of Networks</td><td>";
  $q = 'SELECT count(*) FROM network';
  $r = $db->query($q)->fetchColumn();
  print $r . "</td></tr>\n";

  print "<tr><td>The Number of Genes</td><td>";
  $q = 'SELECT count(*) FROM gene';
  print $db->query($q)->fetchColumn() . "</td></tr>\n";

  print "<tr><td>The Number of Nodes</td><td>";
  $q = 'SELECT count(*) FROM node';
  print $db->query($q)->fetchColumn() . "</td></tr>\n";

  print "<tr><td>The Number of Edges</td><td>";
  $q = 'SELECT count(*) FROM edge';
  print $db->query($q)->fetchColumn() . "</td></tr>\n";

  print "</table>\n";
  print "<br/><br/>\n";
  include("footer.php");
}
/* show_status() =========================================================== */

function put_error($str){
  print '<h1>' . $str . '</h1>';
  include("footer.php");
  die();
}
/* put_error() ============================================================= */

function put_search_box($query){
  print '<div class="search2">';
  print '<form method="GET" action="index.html" name="search">';
  print '<input id="query" type="text" name="q" value="' 
	. htmlspecialchars($query,ENT_QUOTES) 
	. '" placeholder="search query" size="80" /> ';
  print '<input id="submit" type="submit" name="" value="search" />';
  print '</form>';
  print '</div><!-- search_option -->';
}
/* put_search_box() ======================================================== */

function show_network_list($db){

  $q = "SELECT network.id,gene_set.name,gene_set.description,array_set.ref,method.print as method, "
	.      " release.name as release, gene_set_method.name as gsm, method.name as mn, method_param.name as mpn,"
	.      " gene_set.id as gid, data_src.name as src"
	.    " FROM network,dataset,gene_set,array_set,method_param,method,release,release_network,"
	.      " gene_set_method,data_src "
	.    " WHERE network.dataset = dataset.id"
	.       " AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id"
	.       " AND network.method_param = method_param.id AND method_param.method = method.id"
	.       " AND release.id = release_network.release AND release_network.network = network.id"
	.       " AND gene_set_method.id = gene_set.gene_set_method AND data_src.id = array_set.data_src";
  $r = $db->query($q)->fetchAll();

  include("header.php");
  put_search_box("");
  print '<h1>Browse Networks</h1>';
  print '<table>';
  foreach($r as $row){
	print '<tr><td>';
	print '<a href="index.html?t=network&id=' . $row['id'] . '">' 
		  . $row['name'] . " - " . $row['ref'] . " - " . $row['method']
		  . "</a></td>";
		print '<td>' . $row['description'] . '</td></tr>' . "\n";
  }
  print '</table>';
  include("footer.php");

}
/* show_network_list() ===================================================== */

function show_subnetwork_json($db,$get){
  header('Content-type: application/json');

  $node_id = $get['node'];

  /* Retrieve network information */
  $q = 'SELECT network.id,gene_set.name,gene_set.description,'
	.      'array_set.ref,method.print as method,release.name as release,'
	.      'gene_set_method.name as gsm, method.name as mn, method_param.name as mpn,'
	.      'gene_set.id as gid, data_src.name as src'
	.   ' FROM network,dataset,gene_set,array_set,method_param,method,release,release_network,'
	.    ' gene_set_method,data_src '
	.   ' WHERE network.dataset = dataset.id'
	.       " AND dataset.gene_set = gene_set.id AND dataset.array_set = array_set.id"
	.       " AND network.method_param = method_param.id AND method_param.method = method.id"
	.       " AND release.id = release_network.release" 
    .       " AND release_network.network = network.id"
	.       " AND gene_set_method.id = gene_set.gene_set_method"
	.       " AND data_src.id = array_set.data_src"
	.       ' AND network.id=' . $get['network'];
  $r = $db->query($q)->fetchAll();
  $row = $r[0];

  /* Obtain files */
  $q = 'SELECT file.name,format.name as format,format.ext as ext'
	.    ' FROM file,format'
	.    ' WHERE file.format = format.id AND file.network=' . $get['network'];
  $files = $db->query($q)->fetchAll();
  foreach($files as $f){
	$path = 'f/' . $row['release'] . '/' . $row['gsm'] 
	  . '/' . $row['mn'] . '/' . $row['mpn'] . '/'
	  . $f['ext'] . '/' . $f['name'];
	if($f['format'] == 'NODELIST'){
	  $nltxt = $path;
	} else if($f['format'] == 'text'){
	  $txt = $path;
	}
  }

  /* Get the node name. */
  $q = 'SELECT node.name FROM node WHERE node.id = ' . $node_id;
  $node = $db->query($q)->fetchColumn();

  /* Extract Edges */
  $fp = fopen($txt, 'r');
  $token = split("\t", fgets($fp));
  for($i = 0; $i < count($token); $i++){
	if($token[$i] == 'edgeScore'){
	  $pos = $i;
	  break;
	}
  }
  $nodes = array();
  $nodes[$node] = true;
  $ups = array();
  $downs = array();
  while(!feof($fp)){
	$token = split("\t", fgets($fp));
	if($token[0] == $node){
	  $downs[] = array('name' => $token[1], 'score' => $token[$pos]);
	  $nodes[$token[1]] = true;
	} else if($token[1] == $node){
	  $ups[] = array('name' => $token[0], 'score' => $token[$pos]);
	  $nodes[$token[0]] = true;
	}
  }

  /* Get the number of nodes in a network */
  $q = 'SELECT count(*) FROM gene_set_node'
	.    ' WHERE gene_set_node.gene_set=' . $row['gid'];
  $num = $db->query($q)->fetchColumn();

  /* Extract Nodes */
  $fp = fopen($nltxt, 'r');
  $line = fgets($fp);
  $nodes2 = array();
  $rank = 1;
  $max = (float) 0.0;
  $min = (float) $num * $num;
  while(!feof($fp)){
	$token = split("\t", fgets($fp));
	if($max < $token[5]) $max = (float) $token[5];
	if($token[5] < $min) $min = (float) $token[5];
	if($nodes[$token[0]]){
	  $q = 'SELECT id from node where name=' . $db->quote($token[0]);
	  $id = $db->query($q)->fetchColumn();
	  $n = array('id' => $id, 'name' => $token[0], 'betweenness' => round($token[5],3),
				 'norm_betweenness' => (float) $token[5],
				 'hub_rank' => $rank, 'num_children' => (int) $token[2],
				 'num_parents' => (int) $token[3], 'degree' => (int) $token[4]);
	  $nodes2[] = $n;
	  $nodes[$token[0]] = $id;
	}
	$rank++;
  }
  /* Normalize betweenness */
  $range = $max - $min;
  foreach($nodes2 as &$n){
	$n['norm_betweenness'] = ($n['norm_betweenness'] - $min) / $range;
  }

  /* Generate links */
  $links = array();
  foreach($downs as $e){
	$link = array('source' => $node_id,
				  'target' => $nodes[$e['name']],
				  'score' => $e['score']);
	$links[] = $link;
  }
  foreach($ups as $e){
	$link = array('source' => $nodes[$e['name']],
				  'target' => $node_id,
				  'score' => $e['score']);
	$links[] = $link;
  }

  $ret = array();
  $network = array();
  $network['nodes'] = $nodes2;
  $network['links'] = $links;
  $ret['network'] = $network;
  $ret['type'] = 'subnetwork';
  $ret['id'] = $get['network'];
  $ret['name'] = $row['name'] . " - " . $row['ref'] . " - " . $row['method'];
  $ret['condition'] = 'distant 1 neighbor nodes';
  $ret['center'] = $node;
  $ret['center_id'] = $node_id;
  $ret['description'] = $row['description'];
  $ret['number_of_nodes'] = $num;
  $ret['max_betweenness'] = $max;
  $ret['min_betweenness'] = $min;
  $ret['range_betweenness'] = $range;
  echo json_encode($ret);
  //var_dump($ret);
}
/* show_subnetwork_json() ================================================== */

/* End of file.  Copyright (C) 2012-2013 SiGN Project Members */
?>
